# encoding: utf-8

# Get module version
from ._metadata import __version__

# Import key items from module
from .time_constants import *
from .timeout import Timeout
from .throttle import Throttle
from .stopwatch import Stopwatch
from .time_incrementer import TimeIncrementer

# Set default logging handler to avoid "No handler found" warnings.
from logging import NullHandler, getLogger
getLogger(__name__).addHandler(NullHandler())



# encoding: utf-8

import logging_helper
import datetime

logging = logging_helper.setup_logging()

try:
    basestring
except NameError:
    basestring = str


class TimeIncrementer(object):

    DEFAULT_PERIOD = 300

    def __init__(self,
                 start=None,
                 period=DEFAULT_PERIOD,
                 fmt=None):
        u"""
        Returns times based periods above the start time.
        Use next() to advance the 'current' by the period.

        :param start:  datetime object or None
        :param period: timedelta object or a time in seconds
        :param fmt:    A strftime time format string. e.g. "%H:%M"
                       If this is ommitted, datetimes are returned.

        """
        if start is None:
            start = datetime.datetime.now()
        elif isinstance(start, basestring):
            try:
                start = datetime.datetime.strptime(start, fmt)
            except (ValueError, TypeError) as ve:
                logging.exception(ve)
                raise ValueError(u'Start value ("{start}") cannot be converted to {format}'
                                 .format(start=start,
                                         format=fmt))
        self._start = (datetime.datetime.now()
                       if start is None
                       else start)
        self._current = self._start
        try:
            self._period = (period
                            if isinstance(period, datetime.timedelta)
                            else datetime.timedelta(seconds=period))
        except Exception as e:
            logging.exception(e)
            raise ValueError(u'Period value ("{period}") cannot be converted to datetime.timedelta'
                             .format(period=period))
        self._fmt = fmt

    def formatted(self,
                  value):
        if self._fmt:
            return value.strftime(self._fmt)
        else:
            return value

    @property
    def start(self):
        return self.formatted(self._start)

    @property
    def current(self):
        return self.formatted(self._current)

    def next(self):
        self._current += self._period
        return self.current



# encoding: utf-8

import timeit
import datetime
import time
import logging_helper
from .time_constants import (ONE_HOUR,
                             TEN_MINUTES,
                             ONE_MINUTE,
                             TEN_SECONDS,
                             TWENTY_SECONDS,
                             ONE_SECOND)

logging = logging_helper.setup_logging()


class TimersBase(object):

    @property
    def _now(self):

        """
        Returns a time elapsed value. It's just an elapsed time since the
        timer started rather than a clock value
        :return: seconds since timer started (float)
        """

        return timeit.default_timer()


class Timeout(TimersBase):

    def __init__(self,
                 seconds=0,
                 start_time_offset=0,
                 start_in_an_expired_state=False,
                 interrupt_period=None):

        """
        Re-usable timeouts with reporting.
        
        :param seconds:
        :param start_time_offset:
        :param start_in_an_expired_state:
        :param interrupt_period: Allows Timeout.expire to interrupt a wait
                                 after the given time. Must be > 0.
                                 If > seconds, is set to seconds, which
                                 effectively makes it not interruptable!
        """
        if seconds < 0:
            logging.warning(u'Timeout initialised with a negative value. Using 0s instead.')
            seconds = 0
        self.__timeout_length = seconds
        self.__start_time_date = datetime.datetime.now()
        self.__start_time = self._now + start_time_offset

        self.__end_time = self.__start_time
        self.interrupted = False
        if not start_in_an_expired_state:
            self.__end_time += self.__timeout_length

        self.__expired = None
        if interrupt_period is not None:
            if interrupt_period <= 0:
                raise ValueError(u'interrupt_period must be > 0 (value:{i})'
                                 .format(i=self.__interrupt_period))

            self.__interrupt_period = (interrupt_period
                                       if interrupt_period < seconds
                                       else seconds)
        else:
            self.__interrupt_period = None

    @property
    def expired(self):
        if not self.__expired:
            self.__expired = self._now >= self.__end_time

        return self.__expired

    def interrupt(self):
        self.interrupted = True
        self.__expired = True

    def restart(self):
        self.__start_time_date = datetime.datetime.now()
        self.__start_time = self._now
        self.__end_time = self.__start_time + self.__timeout_length
        self.__expired = False
        self.interrupted = False

    @property
    def elapsed_time(self):
        return self._now - self.__start_time

    @property
    def seconds_remaining(self):
        if self.__expired or self.__timeout_length <= 0:
            logging.debug(u'seconds remaining:{sr}'.format(sr=0))
            return 0

        logging.debug(u'end_time:{et}; reference_timer:{tc}'
                      .format(et=self.__end_time,
                              tc=self._now))

        time_remaining = self.__end_time - self._now

        if time_remaining > self.__timeout_length or time_remaining < 0:
            self.__expired = True
            logging.debug(u'seconds remaining:{sr}'.format(sr=0))
            return 0

        logging.debug(u'seconds remaining:{sr}'.format(sr=time_remaining))
        return time_remaining

    @property
    def time_remaining(self):
        return datetime.timedelta(seconds=self.seconds_remaining)

    def __interruptable_wait(self,
                             wait_time):
        while not self.interrupted and wait_time > 0:
            sleep_time = (self.__interrupt_period
                          if self.__interrupt_period < wait_time
                          else wait_time)
            logging.debug(u'non interruptable wait: {sleep_time}'.format(sleep_time=sleep_time))
            time.sleep(sleep_time)
            wait_time -= sleep_time

    def wait(self,
             message=None):

        if message is None:
            if not self.expired:
                wait_time = self.seconds_remaining
                if self.__interrupt_period is None:
                    logging.debug(u'non interruptable wait: {sleep_time}'.format(sleep_time=wait_time))
                    time.sleep(wait_time)
                else:
                    self.__interruptable_wait(wait_time)
        else:
            while not self.expired:
                remaining = self.time_remaining.seconds

                # print rather than log as this gives a live
                # progress to the user for very long intervals
                # but is not relevant in logs.
                print(u'{message} in {minutes}m {seconds}s.'
                      .format(message=message,
                              minutes=remaining / 60,
                              seconds=remaining % 60))

                if remaining > ONE_HOUR:
                    if remaining > ONE_HOUR + TEN_MINUTES:
                        Timeout(ONE_HOUR,
                                interrupt_period=True).wait()
                    else:
                        self.__interruptable_wait(ONE_HOUR - TEN_MINUTES)

                elif remaining > TEN_MINUTES:
                    self.__interruptable_wait(TEN_MINUTES)

                elif remaining > ONE_MINUTE:
                    self.__interruptable_wait(ONE_MINUTE)

                elif remaining < TEN_SECONDS:
                    self.__interruptable_wait(ONE_SECOND)

                else:
                    self.__interruptable_wait(TEN_SECONDS
                                              if remaining >= TWENTY_SECONDS
                                              else remaining - TEN_SECONDS)

            print(u'Wait finished.')



# encoding: utf-8

import datetime


# get the daylight savings/summer/GMT/UTC offset
def calculate_time_offset():
    ten_minutes = datetime.timedelta(minutes=10)
    utc_time = datetime.datetime.utcnow()
    local_time = datetime.datetime.now()
    if local_time.minute == 0:  # could be on boundary so that utc is hh:59:59.xx and local is hh+1:00.00.xx
        utc_time -= ten_minutes
        local_time -= ten_minutes
    utc_time = datetime.datetime.utcnow().replace(minute=0, second=0, microsecond=0)
    local_time = datetime.datetime.now().replace(minute=0, second=0, microsecond=0)
    return local_time - utc_time


try:
    TIME_OFFSET
except NameError:
    TIME_OFFSET = calculate_time_offset()



# encoding: utf-8

import time
import unittest
from timingsutil import Stopwatch
import logging_helper
logging = logging_helper.setup_logging()


class TestConfiguration(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_stopwatch(self):

        stopwatch = Stopwatch()

        for _ in range(3):
            time.sleep(1)
            self.assertEqual(round(stopwatch.lap()), 1)

        self.assertEqual(round(stopwatch.stop()), 3)


if __name__ == u'__main__':
    unittest.main()
